(* direktes Ausdrucken von Encapsulated Postscript-Dateien

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   - Layout: n EPS-Dateien pro Seite passend verkleinert
   J. Rathlev, Jan. 2000
   letzte nderung: Okt. 2020
   *)

unit PSThMain;

interface

uses
  System.SysUtils, Winapi.Messages, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.Samples.Spin, Vcl.Menus,
  PSFontDlg, ShellDirDlg, WinPlot, WinUtils, WinShell, ExtStreams, Vcl.ExtCtrls,
  Vcl.ComCtrls;

const
  Prog = 'Print Postscript Thumbnails';
  Vers = '2.1';

  DefLPT = 3;
  DefRand : TFRect = (Left : 0.5; Top : 0.5; Right : 0.5; Bottom : 0.5);
  PapA4 : TFPoint = (x : 21.0; y : 29.6); (* A4 hoch *)
  DefTopPos = 0.0;    (* Abstand obere Papierkante - Nullpunkt *)
  DefDistance = 1.0;  (* Anbstnde zwischen den Bildern*)
  DefFontTyp = 1;
  DefFontSize = 10;

  cm = 28.35;                     (* pt / cm *)

  ps1 = '%!PS-Adobe';             (* PS-Header *)
  ps2 = '%%Creator: EPS-Thumbs-Print by J. Rathlev';
  ps3 = '%%BoundingBox: 0 0 ';
  ps6 = '%%Pages: (atend)';
  ps4 = '/psdict 120 dict def psdict begin';
  ps5 = '/cm {28.35 mul} def';
  ps7 = '%%Pages: ';
  ps8 = '%%Trailer';
  pshide = '/copypage {} bind def /showpage {} bind def /erasepage {} bind def';
  psshow = '/newpage {showpage} bind def';
  pspage = 'newpage';
  (* Duplexdruck fr HP *)
  psnorm = '[{<</Duplex false>> setpagedevice} stopped cleartomark';
  psdupl = '[{<</Duplex true /Tumble false>> setpagedevice} stopped cleartomark';

  FMax = 1000;

type
  FileListTyp = array [1..FMax] of string;

  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    VerzWahlBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    Label3: TLabel;
    KopEdit: TSpinEdit;
    EndeBtn: TBitBtn;
    DruckBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    InfoBtn: TSpeedButton;
    PrinterRG: TRadioGroup;
    PrtFileEdit: TEdit;
    HochBtn: TSpeedButton;
    DuplexRG: TRadioGroup;
    GroupBox1: TGroupBox;
    ThumbsEdit: TSpinEdit;
    QuerBtn: TSpeedButton;
    RandBtn: TSpeedButton;
    AbstandBtn: TSpeedButton;
    LayoutCB: TComboBox;
    FontBtn: TSpeedButton;
    VerzEdit: TComboBox;
    FileEdit: TComboBox;
    sbProgress: TStatusBar;
    procedure VerzWahlBtnClick(Sender: TObject);
    procedure DruckBtnClick(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PrinterRGClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure VerzEditExit(Sender: TObject);
    procedure FileEditExit(Sender: TObject);
    procedure RandBtnClick(Sender: TObject);
    procedure VerzEditChange(Sender: TObject);
    procedure HochBtnClick(Sender: TObject);
    procedure QuerBtnClick(Sender: TObject);
    procedure AbstandBtnClick(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ArbPfad,
    PrgPfad,PrgName,
    IniName          : string;
    PrtNr            : integer;
    Rand             : TFRect;
    PapSize          : TFPoint;
    TopPos,Distance,
    Scale            : double;
    Querformat       : boolean;
    TextFont         : TPSFont;
    procedure ShowPrinter;
    procedure ShowOrientation;
    function GetBBox (AName             : string;
                      var BBOrg,BBSize  : TFPoint) : boolean;
    procedure CopyPSFile (fsPS           : TWriteFileStream;
                          const PSName   : string;
                          x,y,sc         : double);
    procedure PrintPSText (fsPS          : TWriteFileStream;
                           x,y           : double;
                           const Text    : string);
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

uses Randdlg, SizeDlg, InpFloat, FileUtils, StringUtils, NumberUtils,
  Winapi.ShlObj, Vcl.FileCtrl, System.IniFiles;

{$R *.DFM}

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  FontSekt = 'Font';
  VerzSekt = 'Directories';
  FileSekt = 'Files';

  (* INI-Variablen *)
  IniPrtNr = 'Printer';
  IniPrtFile = 'PrintFile';
  IniLayout = 'Layout';
  IniLCount = 'LayoutCount';
  IniDuplex = 'Duplex';
  IniLeftMarg = 'LeftMargin';
  IniRightMarg = 'RightMargin';
  IniTopMarg = 'TopMargin';
  IniBottomMarg = 'BottomMargin';
  IniTopLine = 'TopPos';
  IniDistance = 'Distance';
  IniFormat = 'Landscape';
  IniHeight = 'PaperHeight';
  IniWidth = 'PaperWidth';
  IniFontType = 'FontType';
  IniFontSize = 'FontSize';
  IniFontStyle = 'FontStyle';
  IniFontColor = 'FontColor';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
  i        : integer;
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(GetDesktopFolder(CSIDL_APPDATA),PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    i:=ReadInteger(CfgSekt,IniLayout,0);
    if (i<0) or (i>=LayoutCB.Items.Count) then i:=0;
    LayoutCB.ItemIndex:=i;
    ThumbsEdit.Value:=ReadInteger(CfgSekt,IniLCount,2);
    i:=ReadInteger(CfgSekt,IniDuplex,0);
    if (i<0) or (i>DuplexRG.Items.Count) then i:=0;
    DuplexRG.ItemIndex:=i;
    PrtNr:=ReadInteger(CfgSekt,IniPrtNr,DefLPT);
    PrtFileEdit.Text:=ReadString(CfgSekt,IniPrtFile,'PPS.PS');
    Querformat:=ReadBool(CfgSekt,IniFormat,false);
    with PapSize do begin
      Y:=ReadFloat(CfgSekt,IniHeight,PapA4.Y);
      X:=ReadFloat(CfgSekt,IniWidth,PapA4.X);
      end;
    with Rand do begin
      Left:=ReadFloat (CfgSekt,IniLeftMarg,DefRand.Left);
      Right:=ReadFloat (CfgSekt,IniRightMarg,DefRand.Right);
      Top:=ReadFloat (CfgSekt,IniTopMarg,DefRand.Top);
      Bottom:=ReadFloat (CfgSekt,IniBottomMarg,DefRand.Bottom);
      end;
    TopPos:=ReadFloat (CfgSekt,IniTopLine,DefTopPos);
    Distance:=ReadFloat (CfgSekt,IniDistance,DefDistance);
    with TextFont do begin
      PSFontTyp:=ReadInteger(FontSekt,IniFontType,DefFontTyp);
      PSSize:=ReadInteger(FontSekt,IniFontSize,DefFontSize);
      PSStyle:=ReadInteger(FontSekt,IniFontStyle,tsNormal);
      PSColor:=TColor(ReadInteger(FontSekt,IniFontColor,integer(clBlack)));
      end;
    Free;
    end;
  if Querformat then QuerBtn.Down:=true else HochBtn.Down:=true;
  LoadHistory(IniName,VerzSekt,VerzEdit);
  LoadHistory(IniName,FileSekt,FileEdit);
  Caption:=Prog;
  end;

{ ------------------------------------------------------------------- }
(* Formular anzeigen *)
procedure THauptForm.FormShow(Sender: TObject);
begin
  VerzEdit.ItemIndex:=0;
  FileEdit.ItemIndex:=0;
  ShowPrinter;
  with sbProgress do Panels[1].Width:=Width-Panels[0].Width-15;
  ShowOrientation;
  end;

{ ------------------------------------------------------------------- }
(* INI-File schreiben *)
procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteInteger(CfgSekt,IniLayout,LayoutCB.ItemIndex);
    WriteInteger(CfgSekt,IniLCount,ThumbsEdit.Value);
    WriteInteger(CfgSekt,IniDuplex,DuplexRG.ItemIndex);
    WriteInteger(CfgSekt,IniPrtNr,PrtNr);
    WriteString(CfgSekt,IniPrtFile,PrtFileEdit.Text);
    WriteBool(CfgSekt,IniFormat,Querformat);
    with PapSize do begin
      WriteString (CfgSekt,IniHeight,FloatToStrF(Y,ffFixed,6,1));
      WriteString (CfgSekt,IniWidth,FloatToStrF(X,ffFixed,6,1));
      end;
    with Rand do begin
      WriteString (CfgSekt,IniLeftMarg,FloatToStrF(Left,ffFixed,5,2));
      WriteString (CfgSekt,IniRightMarg,FloatToStrF(Right,ffFixed,5,2));
      WriteString (CfgSekt,IniTopMarg,FloatToStrF(Top,ffFixed,5,2));
      WriteString (CfgSekt,IniBottomMarg,FloatToStrF(Bottom,ffFixed,5,2));
      end;
    WriteString (CfgSekt,IniTopLine,FloatToStrF(TopPos,ffFixed,5,2));
    with TextFont do begin
      WriteInteger(FontSekt,IniFontType,PSFontTyp);
      WriteInteger(FontSekt,IniFontSize,PSSize);
      WriteInteger(FontSekt,IniFontStyle,PSStyle);
      WriteInteger(FontSekt,IniFontColor,integer(PSColor));
      end;
    Free;
    SaveHistory(IniName,VerzSekt,true,VerzEdit);
    SaveHistory(IniName,FileSekt,true,FileEdit);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
end;

{ ------------------------------------------------------------------- }
(* Drucker-Radio-Buttons anzeigen *)
procedure THauptForm.ShowPrinter;
begin
  if PrtNr=0 then begin
    PrinterRG.ItemIndex:=2;
    PrtFileEdit.Show;
    end
  else begin
    if PrtNr<3 then PrinterRG.ItemIndex:=pred(PrtNr)
    else PrinterRG.ItemIndex:=PrtNr;
    PrtFileEdit.Hide;
    end;
  end;

procedure THauptForm.ShowOrientation;
begin
  with sbProgress do begin
    with Panels[0] do if HochBtn.Down then Text:='Hochformat' else Text:='Querformat';
    Panels[1].Text:='';
    end;
  end;

{ ------------------------------------------------------------------- }
(* Klick auf Printer *)
procedure THauptForm.PrinterRGClick(Sender: TObject);
begin
  with PrinterRG do begin
    if ItemIndex=2 then PrtNr:=0
    else if ItemIndex<2 then PrtNr:=succ(ItemIndex)
      else PrtNr:=ItemIndex;
    end;
  ShowPrinter;
  end;

{ ------------------------------------------------------------------- }
(* Bounding-Bos aus PS-Datei lesen *)
function THauptForm.GetBBox (AName             : string;
                             var BBOrg,BBSize  : TFPoint) : boolean;
var
  ft    : textfile;
  zeile : string;
  i     : integer;
  x1,x2,
  y1,y2 : double;

  (* Integerzahl aus String bis Space *)
  function Cvt (var zeile : string;
                def       : integer) : integer;
  var
    i,ic,n : integer;
  begin
    while (length(zeile)>0) and (zeile[1]=#32) do delete (zeile,1,1);
    if length(zeile)>0 then begin
      i:=pos(#32,zeile); if i=0 then i:=succ(length(zeile));
      val(copy(zeile,1,pred(i)),n,ic); delete(zeile,1,i);
      if ic>0 then Cvt:=Def else Cvt:=n;
      end
    else Cvt:=Def;
    end;

begin
  if FileExists (AName) then begin
    assignfile (ft,AName); reset(ft);
    readln (ft,Zeile); i:=1;
    if copy(zeile,1,2)='%!' then begin
      repeat
        readln (ft,zeile); inc(i);
        until (i>100) or (copy(zeile,1,14)='%%BoundingBox:');
      if i<=100 then begin (* BBox gefunden *)
        delete(zeile,1,14);
        x1:=cvt (zeile,0); y1:=cvt (zeile,0);
        x2:=cvt (zeile,1); y2:=cvt (zeile,1);
        with BBOrg do begin
          x:=x1/cm; y:=y1/cm;
          end;
        with BBSize do begin
          x:=(x2-x1)/cm; y:=(y2-y1)/cm;
          end;
        end
      else begin
        with BBOrg do begin
          x:=0; y:=0;
          end;
        with BBSize do begin
          x:=PapSize.x; y:=PapSize.y;
          end;
        end;
      closefile (ft);
      Result:=true;
      end
    else begin
      closefile (ft);
      Result:=false;
      end;
    end
  else begin
    Result:=false;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Postscript-Datei kopieren *)
procedure THauptForm.CopyPSFile (fsPS           : TWriteFileStream;
                                 const PSName   : string;
                                 x,y,sc         : double);
var
  s,t    : string;
  fsEps  : TReadFileStream;
begin
  try
    fsEps:=TReadFileStream.Create(PSName,fmShareDenyNone);
  except
    ErrorDialog(Prog,'EPS-Datei: '#13+StripPath(PSName,40)+#13' nicht gefunden!');
    Exit;
    end;
  fsPS.WriteAnsiStringLine ('save');
  fsPS.WriteAnsiStringLine (pshide);       (* Showpage ausschalten *)
  (* Nullpunkt verschieben *)
  fsPS.WriteAnsiStringLine (FloatToStrX(x,4,1,'.')+' cm '+FloatToStrX(y,4,1,'.')+' cm translate');
  (* Skalieren *)
  fsPS.WriteAnsiStringLine (FloatToStrX(Sc,6,4,'.')+' '+FloatToStrX(Sc,6,4,'.')+' scale');
  try
    while not fsEps.EOF do begin
      s:=fsEps.ReadAnsiString;
      if (length(s)>0) then begin
        t:=copy(s,1,2);
        if (t='%!') or (t='%%')then s:='% '+s;
        fsPS.WriteAnsiStringLine (s);
        end;
      end;
  finally
    fsEps.Free;
    end;
  fsPS.WriteAnsiStringLine ('restore');
  end;

{ ------------------------------------------------------------------- }
(* Postscript-Text ausgeben *)
procedure THauptForm.PrintPSText (fsPS       : TWriteFileStream;
                                  x,y        : double;
                                  const Text : string);
begin
  fsPS.WriteAnsiStringLine ('save');
  with TextFont do begin
    fsPS.WriteAnsiStringLine ('/'+FontList[PSFontTyp].PSName[PSStyle]+' findfont');
    fsPS.WriteAnsiStringLine (IntToStr(PSSize)+' scalefont setfont');
    end;
  fsPS.WriteAnsiStringLine (FloatToStrX(x,4,1,'.')+' cm '+FloatToStrX(y,4,1,'.')+' cm moveto');
  fsPS.WriteAnsiStringLine ('('+Text+') show');
  fsPS.WriteAnsiStringLine ('restore');
  end;

{ ------------------------------------------------------------------- }
(* Ausdrucken *)
procedure THauptForm.DruckBtnClick(Sender: TObject);
var
  FLPtr    : ^FileListTyp;
  DMaske   : string;
  DirInfo  : TSearchRec;
  PrtFile,
  s,t,pf,
  Name     : string;
  DosError,
  FCnt,j,
  n,i,pn   : integer;
  fsPS     : TWriteFileStream;
  Org,
  BBOrg,
  BBSize,
  Limit    : TFPoint;
  x,y,h    : double;
begin
  Screen.Cursor:=crHourglass;
  n:=ThumbsEdit.Value;
  with Limit do begin
    if Querformat then begin
      x:=PapSize.y; y:=PapSize.x;
      end
    else begin
      x:=PapSize.x; y:=PapSize.y;
      end;
    Y:=Y-Rand.Top;
    X:=X-Rand.Right;
    case LayoutCB.ItemIndex of
    1 : h:=(x-Rand.Left-Distance*(n-1))/n;   // Spalten
    else h:=(y-Rand.Bottom-Distance*n-1)/n;  // Reihen
      end;
    end;
  n:=0;
  if PrtNr=0 then PrtFile:=Erweiter(VerzEdit.Text,PrtFileEdit.Text,'');
  if length(FileEdit.Text)>0 then begin
    if PrtNr=0 then pf:=PrtFile else pf:='LPT'+StrInt(PrtNr,0);
    try
      fsPS:=TWriteFileStream.Create(pf,0);
    except
      ErrorDialog(Prog,'Fehler beim ffnen von '+pf);
      Exit;
      end;
    (* PS-Header *)
    try
      fsPS.WriteAnsiStringLine (ps1);
    except
      on EInOutError do begin
        MessageDlg ('Fehler beim Schreiben nach '+pf,mtError,[mbOK],0);
        Exit;
        end;
      end;
    fsPS.WriteAnsiStringLine (ps2);
    fsPS.WriteAnsiStringLine (ps3+StrInt(round(x*cm),0)+' '+StrInt(round(y*cm),0));
    fsPS.WriteAnsiStringLine (ps6);
    fsPS.WriteAnsiStringLine (ps4);
    if DuplexRG.ItemIndex= 0 then fsPS.WriteAnsiStringLine (psnorm)
    else fsPS.WriteAnsiStringLine (psdupl);
    pn:=1;
    for j:=1 to KopEdit.Value do begin
      t:=FileEdit.Text;
      fsPS.WriteAnsiStringLine ('%%Page: '+StrInt(pn,0)+' '+StrInt(pn,0));
      fsPS.WriteAnsiStringLine (psshow);       (* Showpage umdefinieren *)
      fsPS.WriteAnsiStringLine (ps5);          (* cm umrechnen *)
      if QuerFormat then
        fsPS.WriteAnsiStringLine (FloatToStrX(PapSize.x,4,1,'.')+' cm 0 translate 90 rotate');
      with Org do begin                    // Pos. fr erstes Bild
        x:=Rand.Left; y:=Limit.y-h;
        end;
      repeat
        s:=ReadNxtStr (t,';');
        DMaske:=Erweiter(VerzEdit.Text,s,'');
        New (FLPtr); FCnt:=0;
        DosError:=FindFirst (DMaske,faArchive,DirInfo);
        while DosError=0 do begin
          inc (FCnt);
          FLPtr^[FCnt]:=DirInfo.Name;
          DosError:=FindNext (DirInfo);
          end;
        System.SysUtils.FindClose (DirInfo);
        if FCnt>0 then for i:=1 to FCnt do begin
          Name:=Erweiter(VerzEdit.Text,FLPtr^[i],'');
          if (PrtNr>0) or (Name<>PrtFile) then begin
            if GetBBox (Name,BBOrg,BBSize) then begin
              with Org do PrintPSText (fsPS,x,y-Distance/2,ExtractFilename(Name));
              with BBSize do if x>y then Scale:=h/x else Scale:=h/y;
              x:=Org.x-BBOrg.x*Scale; y:=Org.y-BBOrg.y*Scale;
              with sbProgress.Panels[1] do Text:=MinimizeName(Name,Canvas,Width);
              Application.ProcessMessages;
              inc(n);
              CopyPSFile (fsPS,Name,x,y,Scale);
              with Org do begin
                x:=x+h+Distance;
                if x+h>Limit.x then begin
                  x:=Rand.Left; y:=y-h-Distance;
                  if y<Rand.Bottom-0.1 then begin
                    y:=Limit.y-h;
                    fsPS.WriteAnsiStringLine (pspage);
                    inc(pn);
                    fsPS.WriteAnsiStringLine ('%%Page: '+StrInt(pn,0)+' '+StrInt(pn,0));
                    fsPS.WriteAnsiStringLine (psshow);       (* Showpage umdefinieren *)
                    fsPS.WriteAnsiStringLine (ps5);          (* cm umrechnen *)
                    if QuerFormat then
                      fsPS.WriteAnsiStringLine (FloatToStrX(PapSize.x,4,1,'.')+' cm 0 translate 90 rotate');
                    end;
                  end;
                end;
              end
            else ErrorDialog('Fehler beim Lesen von '+Name);
            end;
          end;
        Dispose (FLPtr);
        until length(t)=0;
      fsPS.WriteAnsiStringLine (pspage);
      inc(pn);
      end;
    try
      fsPS.WriteAnsiStringLine ('end');
      fsPS.WriteAnsiStringLine (ps8);
      fsPS.WriteAnsiStringLine (ps7+IntToStr(pn-1));
      fsPS.WriteAnsiStringLine ('%%EOF');
      fsPS.Free;
    except
      end;
    end;
  sbProgress.Panels[1].Text:=Format('%u Dateien verarbeitet',[n]);
  Screen.Cursor:=crDefault;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog(Prog+' - Vers. '+Vers+#13+
                    ' 2010-2020 Dr. J. Rathlev - D-24222 Schwentinental');
  end;

{ ------------------------------------------------------------------- }
(* Verzeichnis whlen *)
procedure THauptForm.VerzWahlBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=VerzEdit.Text;
  if length(s)=0 then s:=ArbPfad;
  if ShellDirDialog.Execute ('Verzeichnis fr EPS-Dateien',false,true,false,'',s) then begin
    VerzEdit.Text:=s;
    AddToHistory(VerzEdit,s);
    with FileEdit do begin
      Text:='*.eps'; AddToHistory(FileEdit,Text);
      end;
    ShowOrientation;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=VerzEdit.Text;
    Filename:='';
    Title:='Postscript-Dateien auswhlen';
    if Execute then begin
      s:=ExtractFilePath(Files.Strings[0]);
      if s<>VerzEdit.Text then begin
        VerzEdit.Text:=s;
        AddToHistory(VerzEdit,s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddToHistory(FileEdit,Text);
        end;
      ShowOrientation;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Verzeichnis auswhlen *)
procedure THauptForm.VerzEditExit(Sender: TObject);
begin
  AddToHistory(FileEdit,FileEdit.Text);
  ShowOrientation;
  end;

procedure THauptForm.FileEditExit(Sender: TObject);
begin
  AddToHistory(FileEdit,FileEdit.Text);
  ShowOrientation;
  end;

procedure THauptForm.VerzEditChange(Sender: TObject);
begin
  with FileEdit do begin
    Text:='*.eps'; AddToHistory(FileEdit,Text);
    end;
  ShowOrientation;
  end;

procedure THauptForm.RandBtnClick(Sender: TObject);
var
  s : string;
begin
  with PapSize do if Querformat then s:=FloatToStrX(y,4,1,'.')+'x'+FloatToStrX(x,4,1,'.')
  else s:=FloatToStrX(x,4,1,'.')+'x'+FloatToStrX(y,4,1,'.');
  RandDialog.Execute (CursorPos,'Papierformat: '+s,true,Rand,TopPos);
  end;

procedure THauptForm.HochBtnClick(Sender: TObject);
begin
  QuerFormat:=false;
  ShowOrientation;
  end;

procedure THauptForm.QuerBtnClick(Sender: TObject);
begin
  QuerFormat:=true;
  ShowOrientation;
  end;

procedure THauptForm.AbstandBtnClick(Sender: TObject);
begin
  InputFloat (CursorPos,Prog,'Abstnde:','cm','',ffFixed,4,1,0,10,Distance);
  end;

procedure THauptForm.FontBtnClick(Sender: TObject);
begin
  PSFontDialog.Execute(CursorPos,TextFont)
  end;

end.
