(* Delphi Dialog
   Select Printer
   ==============================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2007
   letzte nderung: Okt. 2020
   *)

unit PsPrintDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, HListBox;

type
  TPrintMode = (pmPrinter,pmFile,pmView);

  TPsPrintDialog = class(TForm)
    Panel1: TPanel;
    rbtPrinter: TRadioButton;
    rbtFile: TRadioButton;
    edtFilename: THistoryCombo;
    lblPath: TLabel;
    cbPsView: TCheckBox;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    procedure rbtPrinterClick(Sender: TObject);
    procedure rbtFileClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure LoadFromIni(const AIniName,AIniSection : string);
    function Execute (const PrName,PrPfad : string; var Filename : string;
                      var Mode : TPrintMode) : boolean;
  end;

var
  PsPrintDialog: TPsPrintDialog;

implementation

{$R *.DFM}

uses WinUtils, FileUtils;

{ ------------------------------------------------------------------- }
procedure TPsPrintDialog.LoadFromIni(const AIniName,AIniSection : string);
begin
  with edtFilename do begin
    LoadFromIni(AIniName,AIniSection );
    if Items.Count=0 then Style:=csSimple else Style:=csDropDown;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TPsPrintDialog.rbtPrinterClick(Sender: TObject);
begin
  edtFilename.Enabled:=false;
  cbPsView.Enabled:=false;
  end;

procedure TPsPrintDialog.rbtFileClick(Sender: TObject);
begin
  edtFilename.Enabled:=true;
  cbPsView.Enabled:=true;
  end;

{ ------------------------------------------------------------------- }
function TPsPrintDialog.Execute (const PrName,PrPfad : string; var Filename : string;
                                 var Mode : TPrintMode) : boolean;
begin
  edtFilename.Text:=Filename;
  rbtPrinter.Caption:='auf Drucker '+PrName;
  if length(PrPfad)>0 then lblPath.Caption:='in '+StripPath(PrPfad,40)
  else lblPath.Caption:='';
  if Mode=pmPrinter then begin
    rbtPrinter.Checked:=true;
    edtFilename.Enabled:=false;
    cbPsView.Enabled:=false;
    end
  else begin
    rbtFile.Checked:=true;
    edtFilename.Enabled:=true;
    with cbPsView do begin
      Enabled:=true;
      Checked:=Mode=pmView;
      end;
    end;
  if ShowModal=mrOK then begin
    if rbtPrinter.Checked then Mode:=pmPrinter
    else begin
      if cbPsView.Checked then Mode:=pmView else Mode:=pmFile;
      Filename:=edtFilename.Text;
      end;
    Result:=true;
    end
  else Result:=false;
  end;

end.
