(* Delphi Dialog
   Randeinstellunge fr Ausdruck
   =============================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Apr. 1999 
   last modified: Nov. 2021
   *)

unit Randdlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, NumberEd, WinUtils;

type
  TRandDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    ObenEdit: TFloatEdit;
    Label8: TLabel;
    Label2: TLabel;
    UntenEdit: TFloatEdit;
    Label3: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    EinzugLabel: TLabel;
    EinzugEinheit: TLabel;
    EinzugEdit: TFloatEdit;
    LinksEdit: TFloatEdit;
    RechtsEdit: TFloatEdit;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
  public
    { Public declarations }
    function Execute (APos       : TPoint;
                      ATitle     : string;
                      ShowEzg    : boolean;
                      var Rand   : TFRect;    // in cm
                      var Einzug : double) : boolean; overload;
    function Execute (APos       : TPoint;
                      ATitle     : string;    // in 1/10 mm
                      var Rand   : TRect) : boolean; overload;
    function Execute (APos       : TPoint;
                      ATitle     : string;    // in c
                      var Rand   : TFRect) : boolean; overload;
  end;

var
  RandDialog: TRandDialog;

implementation

{$R *.DFM}

uses GgtDummy;

{------------------------------------------------------------------- }
procedure TRandDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TRandDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

{------------------------------------------------------------------- }
function TRandDialog.Execute (APos       : TPoint;
                              ATitle     : string;
                              ShowEzg    : boolean;
                              var Rand   : TFRect;
                              var Einzug : double) : boolean;
begin
  AdjustFormPosition(Screen,self,APos);
  if length(ATitle)>0 then Caption:=ATitle;
  with Rand do begin
    ObenEdit.Value:=Top;
    UntenEdit.Value:=Bottom;
    LinksEdit.Value:=Left;
    RechtsEdit.Value:=Right;
    end;
  if ShowEzg then begin
    ClientHeight:=165;
    EinzugEdit.Show; EinzugLabel.Show; EinzugEinheit.Show;
    end
  else begin
    ClientHeight:=135;
    EinzugEdit.Hide; EinzugLabel.Hide; EinzugEinheit.Hide;
    end;
  EinzugEdit.value:=Einzug;
  ActiveControl:=ObenEdit;
  if ShowModal=mrOK then begin
    with rand do begin
      Top:=ObenEdit.Value;
      Bottom:=UntenEdit.Value;
      Left:=LinksEdit.Value;
      Right:=RechtsEdit.Value;
      end;
    Einzug:=EinzugEdit.Value;
    Result:=true;
    end
  else Result:=false;
  end;

// Werte in cm
function TRandDialog.Execute (APos       : TPoint;
                              ATitle   : string;
                              var Rand : TFRect) : boolean;
var
  x : double;
begin
  Result:=Execute (APos,ATitle,false,Rand,x);
  end;

// Werte in 1/100 cm
function TRandDialog.Execute (APos     : TPoint;
                              ATitle   : string;
                              var Rand : TRect) : boolean;
var
  mg : TFRect;
  x  : double;
begin
  x:=0;
  with mg do begin
    Top:=Rand.Top/100;
    Bottom:=Rand.Bottom/100;
    Left:=Rand.Left/100;
    Right:=Rand.Right/100;
    end;
  Result:=Execute (APos,ATitle,false,mg,x);
  if Result then with Rand do begin
    Top:=round(mg.Top*100);
    Bottom:=round(mg.Bottom*100);
    Left:=round(mg.Left*100);
    Right:=round(mg.Right*100);
    end;
  end;

end.
